@@ -137,6 +137,15 @@ func (e ConstraintDoesNotExistError) Error() string {
 	return fmt.Sprintf("constraint %q on table %q does not exist", e.Constraint, e.Table)
 }
 
+type ConstraintAlreadyExistsError struct {
+	Table      string
+	Constraint string
+}
+
+func (e ConstraintAlreadyExistsError) Error() string {
+	return fmt.Sprintf("constraint %q on table %q already exists", e.Constraint, e.Table)
+}
+
 type NoUpSQLAllowedError struct{}
 
 func (e NoUpSQLAllowedError) Error() string {
@@ -149,21 +158,12 @@ func (e NoDownSQLAllowedError) Error() string {
 	return "down SQL is not allowed for this operation"
 }
 
-type MultipleAlterColumnChangesError struct {
-	Changes int
+type BackfillNotPossibleError struct {
+	Table string
 }
 
-func (e MultipleAlterColumnChangesError) Error() string {
-	return fmt.Sprintf("alter column operations require exactly one change, found %d", e.Changes)
-}
-
-type InvalidPrimaryKeyError struct {
-	Table  string
-	Fields int
-}
-
-func (e InvalidPrimaryKeyError) Error() string {
-	return fmt.Sprintf("primary key on table %q must be defined on exactly one column, found %d", e.Table, e.Fields)
+func (e BackfillNotPossibleError) Error() string {
+	return fmt.Sprintf("a backfill is required but table %q doesn't have a single column primary key or a UNIQUE, NOT NULL column", e.Table)
 }
 
 type InvalidReplicaIdentityError struct {
@@ -174,3 +174,29 @@ type InvalidReplicaIdentityError struct {
 func (e InvalidReplicaIdentityError) Error() string {
 	return fmt.Sprintf("replica identity on table %q must be one of 'NOTHING', 'DEFAULT', 'INDEX' or 'FULL', found %q", e.Table, e.Identity)
 }
+
+type InvalidOnDeleteSettingError struct {
+	Name    string
+	Setting string
+}
+
+func (e InvalidOnDeleteSettingError) Error() string {
+	return fmt.Sprintf("foreign key %q on_delete setting must be one of: %q, %q, %q, %q or %q, not %q",
+		e.Name,
+		ForeignKeyReferenceOnDeleteNOACTION,
+		ForeignKeyReferenceOnDeleteRESTRICT,
+		ForeignKeyReferenceOnDeleteSETDEFAULT,
+		ForeignKeyReferenceOnDeleteSETNULL,
+		ForeignKeyReferenceOnDeleteCASCADE,
+		e.Setting,
+	)
+}
+
+type AlterColumnNoChangesError struct {
+	Table  string
+	Column string
+}
+
+func (e AlterColumnNoChangesError) Error() string {
+	return fmt.Sprintf("alter column %q on table %q requires at least one change", e.Column, e.Table)
+}
