@@ -4,44 +4,45 @@ package migrations
 
 import (
 	"context"
-	"database/sql"
 	"errors"
 	"fmt"
 	"strings"
 
 	"github.com/lib/pq"
+	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
 
 var _ Operation = (*OpAddColumn)(nil)
 
-func (o *OpAddColumn) Start(ctx context.Context, conn *sql.DB, stateSchema string, s *schema.Schema, cbs ...CallbackFn) error {
+func (o *OpAddColumn) Start(ctx context.Context, conn db.DB, stateSchema string, tr SQLTransformer, s *schema.Schema, cbs ...CallbackFn) (*schema.Table, error) {
 	table := s.GetTable(o.Table)
 
-	if err := addColumn(ctx, conn, *o, table); err != nil {
-		return fmt.Errorf("failed to start add column operation: %w", err)
+	if err := addColumn(ctx, conn, *o, table, tr); err != nil {
+		return nil, fmt.Errorf("failed to start add column operation: %w", err)
 	}
 
 	if o.Column.Comment != nil {
-		if err := addCommentToColumn(ctx, conn, o.Table, TemporaryName(o.Column.Name), *o.Column.Comment); err != nil {
-			return fmt.Errorf("failed to add comment to column: %w", err)
+		if err := addCommentToColumn(ctx, conn, o.Table, TemporaryName(o.Column.Name), o.Column.Comment); err != nil {
+			return nil, fmt.Errorf("failed to add comment to column: %w", err)
 		}
 	}
 
 	if !o.Column.IsNullable() && o.Column.Default == nil {
 		if err := addNotNullConstraint(ctx, conn, o.Table, o.Column.Name, TemporaryName(o.Column.Name)); err != nil {
-			return fmt.Errorf("failed to add not null constraint: %w", err)
+			return nil, fmt.Errorf("failed to add not null constraint: %w", err)
 		}
 	}
 
 	if o.Column.Check != nil {
 		if err := o.addCheckConstraint(ctx, conn); err != nil {
-			return fmt.Errorf("failed to add check constraint: %w", err)
+			return nil, fmt.Errorf("failed to add check constraint: %w", err)
 		}
 	}
 
-	if o.Up != nil {
-		err := createTrigger(ctx, conn, triggerConfig{
+	var tableToBackfill *schema.Table
+	if o.Up != "" {
+		err := createTrigger(ctx, conn, tr, triggerConfig{
 			Name:           TriggerName(o.Table, o.Column.Name),
 			Direction:      TriggerDirectionUp,
 			Columns:        s.GetTable(o.Table).Columns,
@@ -49,24 +50,22 @@ func (o *OpAddColumn) Start(ctx context.Context, conn *sql.DB, stateSchema strin
 			TableName:      o.Table,
 			PhysicalColumn: TemporaryName(o.Column.Name),
 			StateSchema:    stateSchema,
-			SQL:            *o.Up,
+			SQL:            o.Up,
 		})
 		if err != nil {
-			return fmt.Errorf("failed to create trigger: %w", err)
-		}
-		if err := backfill(ctx, conn, table, cbs...); err != nil {
-			return fmt.Errorf("failed to backfill column: %w", err)
+			return nil, fmt.Errorf("failed to create trigger: %w", err)
 		}
+		tableToBackfill = table
 	}
 
 	table.AddColumn(o.Column.Name, schema.Column{
 		Name: TemporaryName(o.Column.Name),
 	})
 
-	return nil
+	return tableToBackfill, nil
 }
 
-func (o *OpAddColumn) Complete(ctx context.Context, conn *sql.DB, s *schema.Schema) error {
+func (o *OpAddColumn) Complete(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
 	tempName := TemporaryName(o.Column.Name)
 
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s RENAME COLUMN %s TO %s",
@@ -119,7 +118,7 @@ func (o *OpAddColumn) Complete(ctx context.Context, conn *sql.DB, s *schema.Sche
 	return err
 }
 
-func (o *OpAddColumn) Rollback(ctx context.Context, conn *sql.DB) error {
+func (o *OpAddColumn) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer) error {
 	tempName := TemporaryName(o.Column.Name)
 
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s DROP COLUMN IF EXISTS %s",
@@ -164,13 +163,15 @@ func (o *OpAddColumn) Validate(ctx context.Context, s *schema.Schema) error {
 		}
 	}
 
-	// Ensure that the column has a primary key defined on exactly one column.
-	pk := table.GetPrimaryKey()
-	if len(pk) != 1 {
-		return InvalidPrimaryKeyError{Table: o.Table, Fields: len(pk)}
+	// Ensure backfill is possible
+	if o.Up != "" {
+		err := checkBackfill(table)
+		if err != nil {
+			return err
+		}
 	}
 
-	if !o.Column.IsNullable() && o.Column.Default == nil && o.Up == nil {
+	if !o.Column.IsNullable() && o.Column.Default == nil && o.Up == "" {
 		return FieldRequiredError{Name: "up"}
 	}
 
@@ -181,7 +182,7 @@ func (o *OpAddColumn) Validate(ctx context.Context, s *schema.Schema) error {
 	return nil
 }
 
-func addColumn(ctx context.Context, conn *sql.DB, o OpAddColumn, t *schema.Table) error {
+func addColumn(ctx context.Context, conn db.DB, o OpAddColumn, t *schema.Table, tr SQLTransformer) error {
 	// don't add non-nullable columns with no default directly
 	// they are handled by:
 	// - adding the column as nullable
@@ -202,14 +203,20 @@ func addColumn(ctx context.Context, conn *sql.DB, o OpAddColumn, t *schema.Table
 	o.Column.Check = nil
 
 	o.Column.Name = TemporaryName(o.Column.Name)
-	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE %s ADD COLUMN %s",
+	colSQL, err := ColumnToSQL(o.Column, tr)
+	if err != nil {
+		return err
+	}
+
+	_, err = conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE %s ADD COLUMN %s",
 		pq.QuoteIdentifier(t.Name),
-		ColumnToSQL(o.Column),
+		colSQL,
 	))
+
 	return err
 }
 
-func addNotNullConstraint(ctx context.Context, conn *sql.DB, table, column, physicalColumn string) error {
+func addNotNullConstraint(ctx context.Context, conn db.DB, table, column, physicalColumn string) error {
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE %s ADD CONSTRAINT %s CHECK (%s IS NOT NULL) NOT VALID",
 		pq.QuoteIdentifier(table),
 		pq.QuoteIdentifier(NotNullConstraintName(column)),
@@ -218,7 +225,7 @@ func addNotNullConstraint(ctx context.Context, conn *sql.DB, table, column, phys
 	return err
 }
 
-func (o *OpAddColumn) addCheckConstraint(ctx context.Context, conn *sql.DB) error {
+func (o *OpAddColumn) addCheckConstraint(ctx context.Context, conn db.DB) error {
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE %s ADD CONSTRAINT %s CHECK (%s) NOT VALID",
 		pq.QuoteIdentifier(o.Table),
 		pq.QuoteIdentifier(o.Column.Check.Name),
