@@ -20,6 +20,7 @@ const (
 	OpNameAlterColumn        OpName = "alter_column"
 	OpNameCreateIndex        OpName = "create_index"
 	OpNameDropIndex          OpName = "drop_index"
+	OpNameRenameConstraint   OpName = "rename_constraint"
 	OpNameDropConstraint     OpName = "drop_constraint"
 	OpNameSetReplicaIdentity OpName = "set_replica_identity"
 	OpRawSQLName             OpName = "sql"
@@ -45,9 +46,11 @@ func ReadMigration(r io.Reader) (*Migration, error) {
 		return nil, err
 	}
 
+	dec := json.NewDecoder(bytes.NewReader(byteValue))
+	dec.DisallowUnknownFields()
+
 	mig := Migration{}
-	err = json.Unmarshal(byteValue, &mig)
-	if err != nil {
+	if err = dec.Decode(&mig); err != nil {
 		return nil, err
 	}
 
@@ -95,6 +98,9 @@ func (v *Operations) UnmarshalJSON(data []byte) error {
 		case OpNameDropColumn:
 			item = &OpDropColumn{}
 
+		case OpNameRenameConstraint:
+			item = &OpRenameConstraint{}
+
 		case OpNameDropConstraint:
 			item = &OpDropConstraint{}
 
@@ -120,7 +126,9 @@ func (v *Operations) UnmarshalJSON(data []byte) error {
 			return fmt.Errorf("unknown migration type: %v", opName)
 		}
 
-		if err := json.Unmarshal(logBody, item); err != nil {
+		dec := json.NewDecoder(bytes.NewReader(logBody))
+		dec.DisallowUnknownFields()
+		if err := dec.Decode(item); err != nil {
 			return fmt.Errorf("decode migration [%v]: %w", opName, err)
 		}
 
@@ -175,6 +183,9 @@ func OperationName(op Operation) OpName {
 	case *OpDropColumn:
 		return OpNameDropColumn
 
+	case *OpRenameConstraint:
+		return OpNameRenameConstraint
+
 	case *OpDropConstraint:
 		return OpNameDropConstraint
 
