@@ -18,12 +18,14 @@ import (
 )
 
 type TestCase struct {
-	name          string
-	migrations    []migrations.Migration
-	wantStartErr  error
-	afterStart    func(t *testing.T, db *sql.DB, schema string)
-	afterComplete func(t *testing.T, db *sql.DB, schema string)
-	afterRollback func(t *testing.T, db *sql.DB, schema string)
+	name            string
+	migrations      []migrations.Migration
+	wantStartErr    error
+	wantRollbackErr error
+	wantCompleteErr error
+	afterStart      func(t *testing.T, db *sql.DB, schema string)
+	afterComplete   func(t *testing.T, db *sql.DB, schema string)
+	afterRollback   func(t *testing.T, db *sql.DB, schema string)
 }
 
 type TestCases []TestCase
@@ -32,12 +34,12 @@ func TestMain(m *testing.M) {
 	testutils.SharedTestMain(m)
 }
 
-func ExecuteTests(t *testing.T, tests TestCases) {
+func ExecuteTests(t *testing.T, tests TestCases, opts ...roll.Option) {
 	testSchema := testutils.TestSchema()
 
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
-			testutils.WithMigratorInSchemaAndConnectionToContainer(t, testSchema, func(mig *roll.Roll, db *sql.DB) {
+			testutils.WithMigratorInSchemaAndConnectionToContainerWithOptions(t, testSchema, opts, func(mig *roll.Roll, db *sql.DB) {
 				ctx := context.Background()
 
 				// run all migrations except the last one
@@ -69,7 +71,14 @@ func ExecuteTests(t *testing.T, tests TestCases) {
 				}
 
 				// roll back the migration
-				if err := mig.Rollback(ctx); err != nil {
+				err = mig.Rollback(ctx)
+				if tt.wantRollbackErr != nil {
+					if !errors.Is(err, tt.wantRollbackErr) {
+						t.Fatalf("Expected error %q, got %q", tt.wantRollbackErr, err)
+					}
+					return
+				}
+				if err != nil {
 					t.Fatalf("Failed to roll back migration: %v", err)
 				}
 
@@ -84,7 +93,14 @@ func ExecuteTests(t *testing.T, tests TestCases) {
 				}
 
 				// complete the last migration
-				if err := mig.Complete(ctx); err != nil {
+				err = mig.Complete(ctx)
+				if tt.wantCompleteErr != nil {
+					if !errors.Is(err, tt.wantCompleteErr) {
+						t.Fatalf("Expected error %q, got %q", tt.wantCompleteErr, err)
+					}
+					return
+				}
+				if err != nil {
 					t.Fatalf("Failed to complete migration: %v", err)
 				}
 
@@ -150,11 +166,18 @@ func ColumnMustHaveType(t *testing.T, db *sql.DB, schema, table, column, expecte
 
 func ColumnMustHaveComment(t *testing.T, db *sql.DB, schema, table, column, expectedComment string) {
 	t.Helper()
-	if !columnHasComment(t, db, schema, table, column, expectedComment) {
+	if !columnHasComment(t, db, schema, table, column, &expectedComment) {
 		t.Fatalf("Expected column %q to have comment %q", column, expectedComment)
 	}
 }
 
+func ColumnMustNotHaveComment(t *testing.T, db *sql.DB, schema, table, column string) {
+	t.Helper()
+	if !columnHasComment(t, db, schema, table, column, nil) {
+		t.Fatalf("Expected column %q to have no comment", column)
+	}
+}
+
 func TableMustHaveComment(t *testing.T, db *sql.DB, schema, table, expectedComment string) {
 	t.Helper()
 	if !tableHasComment(t, db, schema, table, expectedComment) {
@@ -204,16 +227,16 @@ func UniqueConstraintMustExist(t *testing.T, db *sql.DB, schema, table, constrai
 	}
 }
 
-func ValidatedForeignKeyMustExist(t *testing.T, db *sql.DB, schema, table, constraint string) {
+func ValidatedForeignKeyMustExist(t *testing.T, db *sql.DB, schema, table, constraint string, opts ...foreignKeyOnDeleteOpt) {
 	t.Helper()
-	if !foreignKeyExists(t, db, schema, table, constraint, true) {
+	if !foreignKeyExists(t, db, schema, table, constraint, true, opts...) {
 		t.Fatalf("Expected validated foreign key %q to exist", constraint)
 	}
 }
 
-func NotValidatedForeignKeyMustExist(t *testing.T, db *sql.DB, schema, table, constraint string) {
+func NotValidatedForeignKeyMustExist(t *testing.T, db *sql.DB, schema, table, constraint string, opts ...foreignKeyOnDeleteOpt) {
 	t.Helper()
-	if !foreignKeyExists(t, db, schema, table, constraint, false) {
+	if !foreignKeyExists(t, db, schema, table, constraint, false, opts...) {
 		t.Fatalf("Expected not validated foreign key %q to exist", constraint)
 	}
 }
@@ -313,9 +336,24 @@ func uniqueConstraintExists(t *testing.T, db *sql.DB, schema, table, constraint
 	return exists
 }
 
-func foreignKeyExists(t *testing.T, db *sql.DB, schema, table, constraint string, validated bool) bool {
+type foreignKeyOnDeleteOpt func() string
+
+func withOnDeleteCascade() foreignKeyOnDeleteOpt {
+	return func() string { return "c" }
+}
+
+func withOnDeleteSetNull() foreignKeyOnDeleteOpt {
+	return func() string { return "n" }
+}
+
+func foreignKeyExists(t *testing.T, db *sql.DB, schema, table, constraint string, validated bool, opts ...foreignKeyOnDeleteOpt) bool {
 	t.Helper()
 
+	confDelType := "a"
+	for _, opt := range opts {
+		confDelType = opt()
+	}
+
 	var exists bool
 	err := db.QueryRow(`
     SELECT EXISTS (
@@ -325,8 +363,9 @@ func foreignKeyExists(t *testing.T, db *sql.DB, schema, table, constraint string
       AND conname = $2
       AND contype = 'f'
       AND convalidated = $3
+      AND confdeltype = $4 
     )`,
-		fmt.Sprintf("%s.%s", schema, table), constraint, validated).Scan(&exists)
+		fmt.Sprintf("%s.%s", schema, table), constraint, validated, confDelType).Scan(&exists)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -465,10 +504,10 @@ func columnHasType(t *testing.T, db *sql.DB, schema, table, column, expectedType
 	return expectedType == actualType
 }
 
-func columnHasComment(t *testing.T, db *sql.DB, schema, table, column, expectedComment string) bool {
+func columnHasComment(t *testing.T, db *sql.DB, schema, table, column string, expectedComment *string) bool {
 	t.Helper()
 
-	var actualComment string
+	var actualComment *string
 	err := db.QueryRow(fmt.Sprintf(`
     SELECT col_description(
       %[1]s::regclass,
@@ -481,7 +520,10 @@ func columnHasComment(t *testing.T, db *sql.DB, schema, table, column, expectedC
 		t.Fatal(err)
 	}
 
-	return expectedComment == actualComment
+	if expectedComment == nil {
+		return actualComment == nil
+	}
+	return actualComment != nil && *expectedComment == *actualComment
 }
 
 func tableHasComment(t *testing.T, db *sql.DB, schema, table, expectedComment string) bool {
@@ -546,7 +588,11 @@ func insert(t *testing.T, db *sql.DB, schema, version, table string, record map[
 		if i > 0 {
 			recordStr += ", "
 		}
-		recordStr += fmt.Sprintf("'%s'", record[c])
+		if record[c] == "NULL" {
+			recordStr += record[c]
+		} else {
+			recordStr += fmt.Sprintf("'%s'", record[c])
+		}
 	}
 	recordStr += ")"
 
@@ -565,6 +611,24 @@ func MustDelete(t *testing.T, db *sql.DB, schema, version, table string, record
 	}
 }
 
+func MustNotDelete(t *testing.T, db *sql.DB, schema, version, table string, record map[string]string, errorCode string) {
+	t.Helper()
+
+	err := delete(t, db, schema, version, table, record)
+	if err == nil {
+		t.Fatal("Expected DELETE to fail")
+	}
+
+	var pqErr *pq.Error
+	if ok := errors.As(err, &pqErr); ok {
+		if pqErr.Code.Name() != errorCode {
+			t.Fatalf("Expected DELETE to fail with %q, got %q", errorCode, pqErr.Code.Name())
+		}
+	} else {
+		t.Fatalf("DELETE failed with unknown error: %v", err)
+	}
+}
+
 func delete(t *testing.T, db *sql.DB, schema, version, table string, record map[string]string) error {
 	t.Helper()
 	versionSchema := roll.VersionedSchemaName(schema, version)
