@@ -4,37 +4,45 @@ package migrations
 
 import (
 	"context"
-	"database/sql"
 	"fmt"
+	"strings"
 
 	"github.com/lib/pq"
+	"github.com/xataio/pgroll/pkg/db"
 	"github.com/xataio/pgroll/pkg/schema"
 )
 
 var _ Operation = (*OpCreateTable)(nil)
 
-func (o *OpCreateTable) Start(ctx context.Context, conn *sql.DB, stateSchema string, s *schema.Schema, cbs ...CallbackFn) error {
-	tempName := TemporaryName(o.Name)
-	_, err := conn.ExecContext(ctx, fmt.Sprintf("CREATE TABLE %s (%s)",
-		pq.QuoteIdentifier(tempName),
-		columnsToSQL(o.Columns)))
+func (o *OpCreateTable) Start(ctx context.Context, conn db.DB, stateSchema string, tr SQLTransformer, s *schema.Schema, cbs ...CallbackFn) (*schema.Table, error) {
+	// Generate SQL for the columns in the table
+	columnsSQL, err := columnsToSQL(o.Columns, tr)
 	if err != nil {
-		return err
+		return nil, fmt.Errorf("failed to create columns SQL: %w", err)
+	}
+
+	// Create the table under a temporary name
+	tempName := TemporaryName(o.Name)
+	_, err = conn.ExecContext(ctx, fmt.Sprintf("CREATE TABLE %s (%s)",
+		pq.QuoteIdentifier(tempName),
+		columnsSQL))
+	if err != nil {
+		return nil, err
 	}
 
 	// Add comments to any columns that have them
 	for _, col := range o.Columns {
 		if col.Comment != nil {
-			if err := addCommentToColumn(ctx, conn, tempName, col.Name, *col.Comment); err != nil {
-				return fmt.Errorf("failed to add comment to column: %w", err)
+			if err := addCommentToColumn(ctx, conn, tempName, col.Name, col.Comment); err != nil {
+				return nil, fmt.Errorf("failed to add comment to column: %w", err)
 			}
 		}
 	}
 
 	// Add comment to the table itself
 	if o.Comment != nil {
-		if err := addCommentToTable(ctx, conn, tempName, *o.Comment); err != nil {
-			return fmt.Errorf("failed to add comment to table: %w", err)
+		if err := addCommentToTable(ctx, conn, tempName, o.Comment); err != nil {
+			return nil, fmt.Errorf("failed to add comment to table: %w", err)
 		}
 	}
 
@@ -50,10 +58,10 @@ func (o *OpCreateTable) Start(ctx context.Context, conn *sql.DB, stateSchema str
 		Columns: columns,
 	})
 
-	return nil
+	return nil, nil
 }
 
-func (o *OpCreateTable) Complete(ctx context.Context, conn *sql.DB, s *schema.Schema) error {
+func (o *OpCreateTable) Complete(ctx context.Context, conn db.DB, tr SQLTransformer, s *schema.Schema) error {
 	tempName := TemporaryName(o.Name)
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("ALTER TABLE IF EXISTS %s RENAME TO %s",
 		pq.QuoteIdentifier(tempName),
@@ -61,7 +69,7 @@ func (o *OpCreateTable) Complete(ctx context.Context, conn *sql.DB, s *schema.Sc
 	return err
 }
 
-func (o *OpCreateTable) Rollback(ctx context.Context, conn *sql.DB) error {
+func (o *OpCreateTable) Rollback(ctx context.Context, conn db.DB, tr SQLTransformer) error {
 	tempName := TemporaryName(o.Name)
 
 	_, err := conn.ExecContext(ctx, fmt.Sprintf("DROP TABLE IF EXISTS %s",
@@ -103,18 +111,22 @@ func (o *OpCreateTable) Validate(ctx context.Context, s *schema.Schema) error {
 	return nil
 }
 
-func columnsToSQL(cols []Column) string {
+func columnsToSQL(cols []Column, tr SQLTransformer) (string, error) {
 	var sql string
 	for i, col := range cols {
 		if i > 0 {
 			sql += ", "
 		}
-		sql += ColumnToSQL(col)
+		colSQL, err := ColumnToSQL(col, tr)
+		if err != nil {
+			return "", err
+		}
+		sql += colSQL
 	}
-	return sql
+	return sql, nil
 }
 
-func ColumnToSQL(col Column) string {
+func ColumnToSQL(col Column, tr SQLTransformer) (string, error) {
 	sql := fmt.Sprintf("%s %s", pq.QuoteIdentifier(col.Name), col.Type)
 
 	if col.IsPrimaryKey() {
@@ -127,18 +139,28 @@ func ColumnToSQL(col Column) string {
 		sql += " NOT NULL"
 	}
 	if col.Default != nil {
-		sql += fmt.Sprintf(" DEFAULT %s", *col.Default)
+		d, err := tr.TransformSQL(*col.Default)
+		if err != nil {
+			return "", err
+		}
+		sql += fmt.Sprintf(" DEFAULT %s", d)
 	}
 	if col.References != nil {
-		sql += fmt.Sprintf(" CONSTRAINT %s REFERENCES %s(%s)",
+		onDelete := "NO ACTION"
+		if col.References.OnDelete != "" {
+			onDelete = strings.ToUpper(string(col.References.OnDelete))
+		}
+
+		sql += fmt.Sprintf(" CONSTRAINT %s REFERENCES %s(%s) ON DELETE %s",
 			pq.QuoteIdentifier(col.References.Name),
 			pq.QuoteIdentifier(col.References.Table),
-			pq.QuoteIdentifier(col.References.Column))
+			pq.QuoteIdentifier(col.References.Column),
+			onDelete)
 	}
 	if col.Check != nil {
 		sql += fmt.Sprintf(" CONSTRAINT %s CHECK (%s)",
 			pq.QuoteIdentifier(col.Check.Name),
 			col.Check.Constraint)
 	}
-	return sql
+	return sql, nil
 }
