@@ -15,19 +15,31 @@ import (
 
 // Start will apply the required changes to enable supporting the new schema version
 func (m *Roll) Start(ctx context.Context, migration *migrations.Migration, cbs ...migrations.CallbackFn) error {
-	// check if there is an active migration, create one otherwise
-	active, err := m.state.IsActiveMigrationPeriod(ctx, m.schema)
+	tablesToBackfill, err := m.StartDDLOperations(ctx, migration, cbs...)
 	if err != nil {
 		return err
 	}
+
+	// perform backfills for the tables that require it
+	return m.performBackfills(ctx, tablesToBackfill)
+}
+
+// StartDDLOperations performs the DDL operations for the migration. This does
+// not include running backfills for any modified tables.
+func (m *Roll) StartDDLOperations(ctx context.Context, migration *migrations.Migration, cbs ...migrations.CallbackFn) ([]*schema.Table, error) {
+	// check if there is an active migration, create one otherwise
+	active, err := m.state.IsActiveMigrationPeriod(ctx, m.schema)
+	if err != nil {
+		return nil, err
+	}
 	if active {
-		return fmt.Errorf("a migration for schema %q is already in progress", m.schema)
+		return nil, fmt.Errorf("a migration for schema %q is already in progress", m.schema)
 	}
 
 	// create a new active migration (guaranteed to be unique by constraints)
 	newSchema, err := m.state.Start(ctx, m.schema, migration)
 	if err != nil {
-		return fmt.Errorf("unable to start migration: %w", err)
+		return nil, fmt.Errorf("unable to start migration: %w", err)
 	}
 
 	// validate migration
@@ -36,44 +48,72 @@ func (m *Roll) Start(ctx context.Context, migration *migrations.Migration, cbs .
 		if err := m.state.Rollback(ctx, m.schema, migration.Name); err != nil {
 			fmt.Printf("failed to rollback migration: %s\n", err)
 		}
-		return fmt.Errorf("migration is invalid: %w", err)
+		return nil, fmt.Errorf("migration is invalid: %w", err)
+	}
+
+	// run any BeforeStartDDL hooks
+	if m.migrationHooks.BeforeStartDDL != nil {
+		if err := m.migrationHooks.BeforeStartDDL(m); err != nil {
+			return nil, fmt.Errorf("failed to execute BeforeStartDDL hook: %w", err)
+		}
+	}
+
+	// defer execution of any AfterStartDDL hooks
+	if m.migrationHooks.AfterStartDDL != nil {
+		defer m.migrationHooks.AfterStartDDL(m)
 	}
 
 	// execute operations
+	var tablesToBackfill []*schema.Table
 	for _, op := range migration.Operations {
-		err := op.Start(ctx, m.pgConn, m.state.Schema(), newSchema, cbs...)
+		table, err := op.Start(ctx, m.pgConn, m.state.Schema(), m.sqlTransformer, newSchema, cbs...)
 		if err != nil {
 			errRollback := m.Rollback(ctx)
 
-			return errors.Join(
+			return nil, errors.Join(
 				fmt.Errorf("unable to execute start operation: %w", err),
 				errRollback)
 		}
-
+		// refresh schema when the op is isolated and requires a refresh (for example raw sql)
+		// we don't want to refresh the schema if the operation is not isolated as it would
+		// override changes made by other operations
 		if _, ok := op.(migrations.RequiresSchemaRefreshOperation); ok {
-			// refresh schema
-			newSchema, err = m.state.ReadSchema(ctx, m.schema)
-			if err != nil {
-				return fmt.Errorf("unable to refresh schema: %w", err)
+			if isolatedOp, ok := op.(migrations.IsolatedOperation); ok && isolatedOp.IsIsolated() {
+				newSchema, err = m.state.ReadSchema(ctx, m.schema)
+				if err != nil {
+					return nil, fmt.Errorf("unable to refresh schema: %w", err)
+				}
 			}
 		}
+		if table != nil {
+			tablesToBackfill = append(tablesToBackfill, table)
+		}
 	}
 
 	if m.disableVersionSchemas {
 		// skip creating version schemas
-		return nil
+		return tablesToBackfill, nil
 	}
 
+	// create views for the new version
+	if err := m.ensureViews(ctx, newSchema, migration.Name); err != nil {
+		return nil, err
+	}
+
+	return tablesToBackfill, nil
+}
+
+func (m *Roll) ensureViews(ctx context.Context, schema *schema.Schema, version string) error {
 	// create schema for the new version
-	versionSchema := VersionedSchemaName(m.schema, migration.Name)
-	_, err = m.pgConn.ExecContext(ctx, fmt.Sprintf("CREATE SCHEMA IF NOT EXISTS %s", pq.QuoteIdentifier(versionSchema)))
+	versionSchema := VersionedSchemaName(m.schema, version)
+	_, err := m.pgConn.ExecContext(ctx, fmt.Sprintf("CREATE SCHEMA IF NOT EXISTS %s", pq.QuoteIdentifier(versionSchema)))
 	if err != nil {
 		return err
 	}
 
 	// create views in the new schema
-	for name, table := range newSchema.Tables {
-		err = m.createView(ctx, migration.Name, name, table)
+	for name, table := range schema.Tables {
+		err = m.ensureView(ctx, version, name, table)
 		if err != nil {
 			return fmt.Errorf("unable to create view: %w", err)
 		}
@@ -111,12 +151,42 @@ func (m *Roll) Complete(ctx context.Context) error {
 		return fmt.Errorf("unable to read schema: %w", err)
 	}
 
+	// run any BeforeCompleteDDL hooks
+	if m.migrationHooks.BeforeCompleteDDL != nil {
+		if err := m.migrationHooks.BeforeCompleteDDL(m); err != nil {
+			return fmt.Errorf("failed to execute BeforeCompleteDDL hook: %w", err)
+		}
+	}
+
+	// defer execution of any AfterCompleteDDL hooks
+	if m.migrationHooks.AfterCompleteDDL != nil {
+		defer m.migrationHooks.AfterCompleteDDL(m)
+	}
+
 	// execute operations
+	refreshViews := false
 	for _, op := range migration.Operations {
-		err := op.Complete(ctx, m.pgConn, schema)
+		err := op.Complete(ctx, m.pgConn, m.sqlTransformer, schema)
 		if err != nil {
 			return fmt.Errorf("unable to execute complete operation: %w", err)
 		}
+
+		if _, ok := op.(migrations.RequiresSchemaRefreshOperation); ok {
+			refreshViews = true
+		}
+	}
+
+	// recreate views for the new version (if some operations require it, ie SQL)
+	if refreshViews && !m.disableVersionSchemas {
+		schema, err = m.state.ReadSchema(ctx, m.schema)
+		if err != nil {
+			return fmt.Errorf("unable to read schema: %w", err)
+		}
+
+		err = m.ensureViews(ctx, schema, migration.Name)
+		if err != nil {
+			return err
+		}
 	}
 
 	// mark as completed
@@ -146,7 +216,7 @@ func (m *Roll) Rollback(ctx context.Context) error {
 
 	// execute operations
 	for _, op := range migration.Operations {
-		err := op.Rollback(ctx, m.pgConn)
+		err := op.Rollback(ctx, m.pgConn, m.sqlTransformer)
 		if err != nil {
 			return fmt.Errorf("unable to execute rollback operation: %w", err)
 		}
@@ -162,7 +232,7 @@ func (m *Roll) Rollback(ctx context.Context) error {
 }
 
 // create view creates a view for the new version of the schema
-func (m *Roll) createView(ctx context.Context, version, name string, table schema.Table) error {
+func (m *Roll) ensureView(ctx context.Context, version, name string, table schema.Table) error {
 	columns := make([]string, 0, len(table.Columns))
 	for k, v := range table.Columns {
 		columns = append(columns, fmt.Sprintf("%s AS %s", pq.QuoteIdentifier(v.Name), pq.QuoteIdentifier(k)))
@@ -179,7 +249,9 @@ func (m *Roll) createView(ctx context.Context, version, name string, table schem
 	}
 
 	_, err := m.pgConn.ExecContext(ctx,
-		fmt.Sprintf("CREATE OR REPLACE VIEW %s.%s %s AS SELECT %s FROM %s",
+		fmt.Sprintf("BEGIN; DROP VIEW IF EXISTS %s.%s; CREATE VIEW %s.%s %s AS SELECT %s FROM %s; COMMIT",
+			pq.QuoteIdentifier(VersionedSchemaName(m.schema, version)),
+			pq.QuoteIdentifier(name),
 			pq.QuoteIdentifier(VersionedSchemaName(m.schema, version)),
 			pq.QuoteIdentifier(name),
 			withOptions,
@@ -191,6 +263,20 @@ func (m *Roll) createView(ctx context.Context, version, name string, table schem
 	return nil
 }
 
+func (m *Roll) performBackfills(ctx context.Context, tables []*schema.Table) error {
+	for _, table := range tables {
+		if err := migrations.Backfill(ctx, m.pgConn, table); err != nil {
+			errRollback := m.Rollback(ctx)
+
+			return errors.Join(
+				fmt.Errorf("unable to backfill table %q: %w", table.Name, err),
+				errRollback)
+		}
+	}
+
+	return nil
+}
+
 func VersionedSchemaName(schema string, version string) string {
 	return schema + "_" + version
 }
