@@ -5,12 +5,12 @@ package roll_test
 import (
 	"context"
 	"database/sql"
-	"errors"
 	"fmt"
 	"testing"
+	"time"
 
-	"github.com/lib/pq"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 	"github.com/xataio/pgroll/pkg/migrations"
 	"github.com/xataio/pgroll/pkg/roll"
 	"github.com/xataio/pgroll/pkg/state"
@@ -82,21 +82,6 @@ func TestDisabledSchemaManagement(t *testing.T) {
 	})
 }
 
-func schemaExists(t *testing.T, db *sql.DB, schema string) bool {
-	t.Helper()
-	var exists bool
-	err := db.QueryRow(`
-	SELECT EXISTS(
-		SELECT 1
-		FROM pg_catalog.pg_namespace
-		WHERE nspname = $1
-	)`, schema).Scan(&exists)
-	if err != nil {
-		t.Fatal(err)
-	}
-	return exists
-}
-
 func TestPreviousVersionIsDroppedAfterMigrationCompletion(t *testing.T) {
 	t.Parallel()
 
@@ -193,6 +178,85 @@ func TestSchemaIsDroppedAfterMigrationRollback(t *testing.T) {
 	})
 }
 
+func TestRollbackOnMigrationStartFailure(t *testing.T) {
+	t.Parallel()
+
+	t.Run("when the DDL phase fails", func(t *testing.T) {
+		t.Parallel()
+
+		testutils.WithMigratorAndConnectionToContainer(t, func(mig *roll.Roll, db *sql.DB) {
+			ctx := context.Background()
+
+			// start a migration that will fail during the DDL phase
+			err := mig.Start(ctx, &migrations.Migration{
+				Name: "01_create_table",
+				Operations: migrations.Operations{
+					&migrations.OpCreateTable{
+						Name: "table1",
+						Columns: []migrations.Column{
+							{
+								Name: "id",
+								Type: "invalid",
+							},
+						},
+					},
+				},
+			})
+			assert.Error(t, err)
+
+			// ensure that there is no active migration
+			status, err := mig.Status(ctx, "public")
+			assert.NoError(t, err)
+			assert.Equal(t, state.NoneMigrationStatus, status.Status)
+		})
+	})
+
+	t.Run("when the backfill phase fails", func(t *testing.T) {
+		t.Parallel()
+
+		testutils.WithMigratorAndConnectionToContainer(t, func(mig *roll.Roll, db *sql.DB) {
+			ctx := context.Background()
+
+			// run an initial migration to create the table
+			err := mig.Start(ctx, &migrations.Migration{
+				Name:       "01_create_table",
+				Operations: migrations.Operations{createTableOp("table1")},
+			})
+			assert.NoError(t, err)
+
+			// complete the migration
+			err = mig.Complete(ctx)
+			assert.NoError(t, err)
+
+			// insert some data into the table
+			_, err = db.ExecContext(ctx, "INSERT INTO table1 (id, name) VALUES (1, 'alice'), (2, 'bob')")
+			assert.NoError(t, err)
+
+			// Start a migration that will fail during the backfill phase
+			// Change the type of the `name` column but provide invalid up and down SQL
+			err = mig.Start(ctx, &migrations.Migration{
+				Name: "02_add_column",
+				Operations: migrations.Operations{
+					&migrations.OpAlterColumn{
+						Table:  "table1",
+						Column: "name",
+						Type:   ptr("text"),
+						Up:     "invalid",
+						Down:   "invalid",
+					},
+				},
+			})
+			assert.Error(t, err)
+
+			// Ensure that there is no active migration
+			status, err := mig.Status(ctx, "public")
+			assert.NoError(t, err)
+			assert.Equal(t, "01_create_table", status.Version)
+			assert.Equal(t, state.CompleteMigrationStatus, status.Status)
+		})
+	})
+}
+
 func TestSchemaOptionIsRespected(t *testing.T) {
 	t.Parallel()
 
@@ -248,57 +312,48 @@ func TestSchemaOptionIsRespected(t *testing.T) {
 	})
 }
 
-func TestLockTimeoutIsEnforced(t *testing.T) {
+func TestMigrationDDLIsRetriedOnLockTimeouts(t *testing.T) {
 	t.Parallel()
 
-	testutils.WithMigratorInSchemaAndConnectionToContainerWithOptions(t, "public", []roll.Option{roll.WithLockTimeoutMs(100)}, func(mig *roll.Roll, db *sql.DB) {
+	testutils.WithMigratorInSchemaAndConnectionToContainerWithOptions(t, "public", []roll.Option{roll.WithLockTimeoutMs(50)}, func(mig *roll.Roll, db *sql.DB) {
 		ctx := context.Background()
 
-		// Start a create table migration
-		err := mig.Start(ctx, &migrations.Migration{
-			Name:       "01_create_table",
-			Operations: migrations.Operations{createTableOp("table1")},
-		})
-		if err != nil {
-			t.Fatalf("Failed to start migration: %v", err)
-		}
+		// Create a table
+		_, err := db.ExecContext(ctx, "CREATE TABLE table1 (id integer, name text)")
+		require.NoError(t, err)
 
-		// Complete the create table migration
-		if err := mig.Complete(ctx); err != nil {
-			t.Fatalf("Failed to complete migration: %v", err)
-		}
+		// Start a goroutine which takes an ACCESS_EXCLUSIVE lock on the table for
+		// two seconds
+		errCh := make(chan error)
+		go func() {
+			tx, err := db.Begin()
+			if err != nil {
+				errCh <- err
+			}
 
-		// Start a transaction and take an ACCESS_EXCLUSIVE lock on the table
-		// Don't commit the transaction so that the lock is held indefinitely
-		tx, err := db.Begin()
-		if err != nil {
-			t.Fatalf("Failed to start transaction: %v", err)
-		}
-		t.Cleanup(func() {
+			if _, err := tx.ExecContext(ctx, "LOCK TABLE table1 IN ACCESS EXCLUSIVE MODE"); err != nil {
+				errCh <- err
+			}
+			errCh <- nil
+
+			// Sleep for two seconds to hold the lock
+			time.Sleep(2 * time.Second)
+
+			// Commit the transaction
 			tx.Commit()
-		})
-		if _, err := tx.ExecContext(ctx, "LOCK TABLE table1 IN ACCESS EXCLUSIVE MODE"); err != nil {
-			t.Fatalf("Failed to take ACCESS_EXCLUSIVE lock on table: %v", err)
-		}
+		}()
 
-		// Attempt to run a second migration on the table while the lock is held
-		// The migration should fail due to a lock timeout error
+		// Wait for lock to be taken
+		err = <-errCh
+		require.NoError(t, err)
+
+		// Attempt to start a second migration on the table while the lock is held.
+		// The migration should eventually succeed after the lock is released
 		err = mig.Start(ctx, &migrations.Migration{
-			Name:       "02_create_table",
+			Name:       "01_add_column",
 			Operations: migrations.Operations{addColumnOp("table1")},
 		})
-		if err == nil {
-			t.Fatalf("Expected migration to fail due to lock timeout")
-		}
-		if err != nil {
-			pqErr := &pq.Error{}
-			if ok := errors.As(err, &pqErr); !ok {
-				t.Fatalf("Migration failed with unexpected error: %v", err)
-			}
-			if pqErr.Code != "55P03" { // Lock not available error code
-				t.Fatalf("Migration failed with unexpected error: %v", err)
-			}
-		}
+		require.NoError(t, err)
 	})
 }
 
@@ -488,6 +543,174 @@ func TestRoleIsRespected(t *testing.T) {
 	})
 }
 
+func TestMigrationHooksAreInvoked(t *testing.T) {
+	t.Parallel()
+
+	options := []roll.Option{roll.WithMigrationHooks(roll.MigrationHooks{
+		BeforeStartDDL: func(m *roll.Roll) error {
+			_, err := m.PgConn().ExecContext(context.Background(), "CREATE TABLE before_start_ddl (id integer)")
+			return err
+		},
+		AfterStartDDL: func(m *roll.Roll) error {
+			_, err := m.PgConn().ExecContext(context.Background(), "CREATE TABLE after_start_ddl (id integer)")
+			return err
+		},
+		BeforeCompleteDDL: func(m *roll.Roll) error {
+			_, err := m.PgConn().ExecContext(context.Background(), "CREATE TABLE before_complete_ddl (id integer)")
+			return err
+		},
+		AfterCompleteDDL: func(m *roll.Roll) error {
+			_, err := m.PgConn().ExecContext(context.Background(), "CREATE TABLE after_complete_ddl (id integer)")
+			return err
+		},
+	})}
+
+	testutils.WithMigratorInSchemaAndConnectionToContainerWithOptions(t, "public", options, func(mig *roll.Roll, db *sql.DB) {
+		ctx := context.Background()
+
+		// Start a create table migration
+		err := mig.Start(ctx, &migrations.Migration{
+			Name:       "01_create_table",
+			Operations: migrations.Operations{createTableOp("table1")},
+		})
+		assert.NoError(t, err)
+
+		// Ensure that both the before_start_ddl and after_start_ddl tables were created
+		assert.True(t, tableExists(t, db, "public", "before_start_ddl"))
+		assert.True(t, tableExists(t, db, "public", "after_start_ddl"))
+
+		// Complete the migration
+		err = mig.Complete(ctx)
+		assert.NoError(t, err)
+
+		// Ensure that both the before_complete_ddl and after_complete_ddl tables were created
+		assert.True(t, tableExists(t, db, "public", "before_complete_ddl"))
+		assert.True(t, tableExists(t, db, "public", "after_complete_ddl"))
+	})
+}
+
+func TestRollSchemaMethodReturnsCorrectSchema(t *testing.T) {
+	t.Parallel()
+
+	t.Run("when the schema is public", func(t *testing.T) {
+		testutils.WithMigratorInSchemaAndConnectionToContainer(t, "public", func(mig *roll.Roll, _ *sql.DB) {
+			assert.Equal(t, "public", mig.Schema())
+		})
+	})
+
+	t.Run("when the schema is non-public", func(t *testing.T) {
+		testutils.WithMigratorInSchemaAndConnectionToContainer(t, "apples", func(mig *roll.Roll, _ *sql.DB) {
+			assert.Equal(t, "apples", mig.Schema())
+		})
+	})
+}
+
+func TestSQLTransformerOptionIsUsedWhenCreatingTriggers(t *testing.T) {
+	t.Parallel()
+
+	t.Run("when the SQL transformer is used to rewrite SQL", func(t *testing.T) {
+		t.Parallel()
+
+		sqlTransformer := testutils.NewMockSQLTransformer(map[string]string{
+			"'apples'": "'rewritten'",
+		})
+		opts := []roll.Option{roll.WithSQLTransformer(sqlTransformer)}
+
+		testutils.WithMigratorAndConnectionToContainerWithOptions(t, opts, func(mig *roll.Roll, db *sql.DB) {
+			ctx := context.Background()
+
+			// Start a create table migration
+			err := mig.Start(ctx, &migrations.Migration{
+				Name:       "01_create_table",
+				Operations: migrations.Operations{createTableOp("table1")},
+			})
+			require.NoError(t, err)
+
+			// Complete the migration
+			err = mig.Complete(ctx)
+			require.NoError(t, err)
+
+			// Insert some data
+			_, err = db.ExecContext(ctx, "INSERT INTO table1 (id, name) VALUES (1, 'alice'), (2, 'bob')")
+			require.NoError(t, err)
+
+			// Start an add column migration that requires a backfill
+			err = mig.Start(ctx, &migrations.Migration{
+				Name: "02_add_column",
+				Operations: migrations.Operations{
+					&migrations.OpAddColumn{
+						Table: "table1",
+						Up:    "'apples'",
+						Column: migrations.Column{
+							Name:     "description",
+							Type:     "text",
+							Nullable: ptr(false),
+						},
+					},
+				},
+			})
+			require.NoError(t, err)
+
+			// Complete the migration
+			err = mig.Complete(ctx)
+			require.NoError(t, err)
+
+			// Ensure that the backfill used the SQL rewritten by the transformer
+			rows := MustSelect(t, db, "public", "02_add_column", "table1")
+			assert.Equal(t, []map[string]any{
+				{"id": 1, "name": "alice", "description": "rewritten"},
+				{"id": 2, "name": "bob", "description": "rewritten"},
+			}, rows)
+		})
+	})
+
+	t.Run("when the SQL transformer returns an error", func(t *testing.T) {
+		t.Parallel()
+
+		sqlTransformer := testutils.NewMockSQLTransformer(map[string]string{
+			"'apples'": testutils.MockSQLTransformerError,
+		})
+		opts := []roll.Option{roll.WithSQLTransformer(sqlTransformer)}
+
+		testutils.WithMigratorAndConnectionToContainerWithOptions(t, opts, func(mig *roll.Roll, db *sql.DB) {
+			ctx := context.Background()
+
+			// Start a create table migration
+			err := mig.Start(ctx, &migrations.Migration{
+				Name:       "01_create_table",
+				Operations: migrations.Operations{createTableOp("table1")},
+			})
+			require.NoError(t, err)
+
+			// Complete the migration
+			err = mig.Complete(ctx)
+			require.NoError(t, err)
+
+			// Insert some data
+			_, err = db.ExecContext(ctx, "INSERT INTO table1 (id, name) VALUES (1, 'alice'), (2, 'bob')")
+			require.NoError(t, err)
+
+			// Start an add column migration that requires a backfill
+			err = mig.Start(ctx, &migrations.Migration{
+				Name: "02_add_column",
+				Operations: migrations.Operations{
+					&migrations.OpAddColumn{
+						Table: "table1",
+						Up:    "'apples'",
+						Column: migrations.Column{
+							Name:     "description",
+							Type:     "text",
+							Nullable: ptr(false),
+						},
+					},
+				},
+			})
+			// Ensure that the start phase has failed with a SQL transformer error
+			require.ErrorIs(t, err, testutils.ErrMockSQLTransformer)
+		})
+	})
+}
+
 func createTableOp(tableName string) *migrations.OpCreateTable {
 	return &migrations.OpCreateTable{
 		Name: tableName,
@@ -560,6 +783,40 @@ func MustSelect(t *testing.T, db *sql.DB, schema, version, table string) []map[s
 	return res
 }
 
+func schemaExists(t *testing.T, db *sql.DB, schema string) bool {
+	t.Helper()
+	var exists bool
+	err := db.QueryRow(`
+	SELECT EXISTS(
+		SELECT 1
+		FROM pg_catalog.pg_namespace
+		WHERE nspname = $1
+	)`, schema).Scan(&exists)
+	if err != nil {
+		t.Fatal(err)
+	}
+	return exists
+}
+
+func tableExists(t *testing.T, db *sql.DB, schema, table string) bool {
+	t.Helper()
+
+	var exists bool
+	err := db.QueryRow(`
+		SELECT EXISTS(
+      SELECT 1
+			FROM pg_catalog.pg_tables
+			WHERE schemaname = $1
+      AND tablename = $2
+    )`,
+		schema, table).Scan(&exists)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	return exists
+}
+
 func ptr[T any](v T) *T {
 	return &v
 }
