@@ -73,18 +73,18 @@ STABLE;
 CREATE OR REPLACE FUNCTION %[1]s.previous_version(schemaname NAME) RETURNS text
 AS $$
   WITH RECURSIVE find_ancestor AS (
-    SELECT schema, name, parent, migration_type FROM pgroll.migrations 
+    SELECT schema, name, parent, migration_type FROM %[1]s.migrations
       WHERE name = (SELECT %[1]s.latest_version(schemaname)) AND schema = schemaname
 
     UNION ALL
 
-    SELECT m.schema, m.name, m.parent, m.migration_type FROM pgroll.migrations m
+    SELECT m.schema, m.name, m.parent, m.migration_type FROM %[1]s.migrations m
       INNER JOIN find_ancestor fa ON fa.parent = m.name AND fa.schema = m.schema
       WHERE m.migration_type = 'inferred'
   )
   SELECT a.parent
   FROM find_ancestor AS a
-  JOIN pgroll.migrations AS b ON a.parent = b.name AND a.schema = b.schema
+  JOIN %[1]s.migrations AS b ON a.parent = b.name AND a.schema = b.schema
   WHERE b.migration_type = 'pgroll';
 $$
 LANGUAGE SQL
@@ -131,7 +131,7 @@ BEGIN
 								SELECT 1
 								FROM pg_constraint
 								WHERE conrelid = attr.attrelid
-								AND conkey::int[] @> ARRAY[attr.attnum::int]
+								AND ARRAY[attr.attnum::int] @> conkey::int[]
 								AND contype = 'u'
 							) OR EXISTS (
 								SELECT 1
@@ -139,7 +139,7 @@ BEGIN
 								JOIN pg_class ON pg_class.oid = pg_index.indexrelid
 								WHERE indrelid = attr.attrelid
 								AND indisunique
-								AND pg_index.indkey::int[] @> ARRAY[attr.attnum::int]
+								AND ARRAY[attr.attnum::int] @> pg_index.indkey::int[]
 							)) AS unique
 						FROM
 							pg_attribute AS attr
@@ -174,7 +174,7 @@ BEGIN
 				  )), '{}'::json)
 				  FROM (
 				    SELECT 
-				      reverse(split_part(reverse(pi.indexrelid::regclass::text), '.', 1)) as name,
+				      replace(reverse(split_part(reverse(pi.indexrelid::regclass::text), '.', 1)), '"', '') as name,
 				      pi.indisunique,
 				      array_agg(a.attname) AS columns
 				    FROM pg_index pi
@@ -223,21 +223,29 @@ BEGIN
 						'name', fk_details.conname,
 						'columns', fk_details.columns,
 						'referencedTable', fk_details.referencedTable,
-						'referencedColumns', fk_details.referencedColumns
+						'referencedColumns', fk_details.referencedColumns,
+						'onDelete', fk_details.onDelete
 					)), '{}'::json)
 					FROM (
 						SELECT
 							fk_constraint.conname,
 							array_agg(fk_attr.attname ORDER BY fk_constraint.conkey::int[]) AS columns,
 							fk_cl.relname AS referencedTable,
-							array_agg(ref_attr.attname ORDER BY fk_constraint.confkey::int[]) AS referencedColumns
+							array_agg(ref_attr.attname ORDER BY fk_constraint.confkey::int[]) AS referencedColumns,
+							CASE
+							  WHEN fk_constraint.confdeltype = 'a' THEN 'NO ACTION'
+							  WHEN fk_constraint.confdeltype = 'r' THEN 'RESTRICT'
+							  WHEN fk_constraint.confdeltype = 'c' THEN 'CASCADE'
+							  WHEN fk_constraint.confdeltype = 'd' THEN 'SET DEFAULT'
+							  WHEN fk_constraint.confdeltype = 'n' THEN 'SET NULL'
+							END as onDelete
 						FROM pg_constraint AS fk_constraint
 						INNER JOIN pg_class fk_cl ON fk_constraint.confrelid = fk_cl.oid
 						INNER JOIN pg_attribute fk_attr ON fk_attr.attrelid = fk_constraint.conrelid AND fk_attr.attnum = ANY(fk_constraint.conkey)
 						INNER JOIN pg_attribute ref_attr ON ref_attr.attrelid = fk_constraint.confrelid AND ref_attr.attnum = ANY(fk_constraint.confkey)
 						WHERE fk_constraint.conrelid = t.oid
 						AND fk_constraint.contype = 'f'
-						GROUP BY fk_constraint.conname, fk_cl.relname
+						GROUP BY fk_constraint.conname, fk_cl.relname, fk_constraint.confdeltype
 					) AS fk_details
 				)
 			)), '{}'::json) FROM pg_class AS t
@@ -268,7 +276,7 @@ BEGIN
 		RETURN;
 	END IF;
 
-	IF tg_event = 'sql_drop' THEN
+	IF tg_event = 'sql_drop' and tg_tag != 'ALTER TABLE' THEN
 		-- Guess the schema from drop commands
 		SELECT schema_name INTO schemaname FROM pg_catalog.pg_event_trigger_dropped_objects() WHERE schema_name IS NOT NULL;
 
@@ -324,7 +332,6 @@ CREATE EVENT TRIGGER pg_roll_handle_ddl ON ddl_command_end
 DROP EVENT TRIGGER IF EXISTS pg_roll_handle_drop;
 CREATE EVENT TRIGGER pg_roll_handle_drop ON sql_drop
 	EXECUTE FUNCTION %[1]s.raw_migration();
-
 `
 
 type State struct {
@@ -333,7 +340,14 @@ type State struct {
 }
 
 func New(ctx context.Context, pgURL, stateSchema string) (*State, error) {
-	conn, err := sql.Open("postgres", pgURL)
+	dsn, err := pq.ParseURL(pgURL)
+	if err != nil {
+		dsn = pgURL
+	}
+
+	dsn += " search_path=" + stateSchema
+
+	conn, err := sql.Open("postgres", dsn)
 	if err != nil {
 		return nil, err
 	}
@@ -354,11 +368,28 @@ func New(ctx context.Context, pgURL, stateSchema string) (*State, error) {
 }
 
 func (s *State) Init(ctx context.Context) error {
-	// ensure pgroll internal tables exist
-	// TODO: eventually use migrations for this instead of hardcoding
-	_, err := s.pgConn.ExecContext(ctx, fmt.Sprintf(sqlInit, pq.QuoteIdentifier(s.schema)))
+	tx, err := s.pgConn.Begin()
+	if err != nil {
+		return err
+	}
+	defer tx.Rollback()
 
-	return err
+	// Try to obtain an advisory lock.
+	// The key is an arbitrary number, used to distinguish the lock from other locks.
+	// The lock is automatically released when the transaction is committed or rolled back.
+	const key int64 = 0x2c03057fb9525b
+	_, err = tx.ExecContext(ctx, "SELECT pg_advisory_xact_lock($1)", key)
+	if err != nil {
+		return err
+	}
+
+	// Perform pgroll state initialization
+	_, err = tx.ExecContext(ctx, fmt.Sprintf(sqlInit, pq.QuoteIdentifier(s.schema)))
+	if err != nil {
+		return err
+	}
+
+	return tx.Commit()
 }
 
 func (s *State) Close() error {
